@@ -7,28 +7,33 @@
 //! **Currently implemented boiler plate for:**
 //!
 //! * **actix-web**
+//! * **rocket** `version >=0.5.0-rc.1`
 //!
-//! Serving Swagger UI is framework independant thus this crate also supports serving the Swagger UI in with
+//! Serving Swagger UI is framework independant thus this crate also supports serving the Swagger UI with
 //! other frameworks as well. With other frameworks there is bit more manual implementation to be done. See
-//! more details at [`serve`].
+//! more details at [`serve`] or [`examples`][examples].
+//!
+//! [examples]: <https://github.com/juhaku/utoipa/tree/master/examples>
 //!
 //! # Features
 //!
 //! * **actix-web** Enables actix-web integration with pre-configured SwaggerUI service factory allowing
 //!   users to use the Swagger UI without a hazzle.
+//! * **rocket** Enables rocket integration with with pre-configured routes for serving the Swagger UI
+//!   and api doc without a hazzle.
 //!
 //! # Install
 //!
 //! Use only the raw types without any boiler plate implementation.
 //! ```text
 //! [dependencies]
-//! utoipa-swagger-ui = "0.2.0"
+//! utoipa-swagger-ui = "0.3.0"
 //!
 //! ```
 //! Enable actix-web framework with Swagger UI you could define the dependency as follows.
 //! ```text
 //! [dependencies]
-//! utoipa-swagger-ui = { version = "0.2.0", features = ["actix-web"] }
+//! utoipa-swagger-ui = { version = "0.3.0", features = ["actix-web"] }
 //! ```
 //!
 //! **Note!** Also remember that you already have defined `utoipa` dependency in your `Cargo.toml`
@@ -54,16 +59,55 @@
 //!     .bind((Ipv4Addr::UNSPECIFIED, 8989)).unwrap()
 //!     .run();
 //! ```
+//!
+//! Serve Swagger UI with api doc via rocket [^rocket]
+//! ```no_run
+//! # use rocket::{Build, Rocket};
+//! # use utoipa_swagger_ui::SwaggerUi;
+//! # use utoipa::OpenApi;
+//! #[rocket::launch]
+//! fn rocket() -> Rocket<Build> {
+//! #
+//! #     #[derive(OpenApi)]
+//! #     #[openapi()]
+//! #     struct ApiDoc;
+//! #
+//!     rocket::build()
+//!         .mount(
+//!             "/",
+//!             SwaggerUi::new("/swagger-ui/<_..>").url("/api-doc/openapi.json", ApiDoc::openapi()),
+//!         )
+//! }
+//! ```
+//!
 //! [^actix]: **actix-web** feature need to be enabled.
+//!
+//! [^rocket]: **rocket** feature need to be enabled.
 use std::{borrow::Cow, error::Error, sync::Arc};
 
 #[cfg(feature = "actix-web")]
 use actix_web::{
-    dev::HttpServiceFactory, guard::Get, web, web::Data, HttpResponse, Resource, Responder,
+    dev::HttpServiceFactory, guard::Get, web, web::Data, HttpResponse, Resource,
+    Responder as ActixResponder,
+};
+
+#[cfg(feature = "rocket")]
+use std::io::Cursor;
+
+#[cfg(feature = "rocket")]
+use rocket::{
+    http::{Header, Status},
+    response::{
+        status::{self, NotFound},
+        Responder as RocketResponder,
+    },
+    route::{Handler, Outcome},
+    serde::json::Json,
+    Data as RocketData, Request, Response, Route,
 };
 
 use rust_embed::RustEmbed;
-#[cfg(feature = "actix-web")]
+#[cfg(any(feature = "actix-web", feature = "rocket"))]
 use utoipa::openapi::OpenApi;
 
 #[derive(RustEmbed)]
@@ -76,13 +120,13 @@ struct SwaggerUiDist;
 /// [^actix]: **actix-web** feature need to be enabled.
 #[non_exhaustive]
 #[derive(Clone)]
-#[cfg(feature = "actix-web")]
+#[cfg(any(feature = "actix-web", feature = "rocket"))]
 pub struct SwaggerUi {
     path: Cow<'static, str>,
     urls: Vec<(Url<'static>, OpenApi)>,
 }
 
-#[cfg(feature = "actix-web")]
+#[cfg(any(feature = "actix-web", feature = "rocket"))]
 impl SwaggerUi {
     /// Create a new [`SwaggerUi`] for given path.
     ///
@@ -197,7 +241,7 @@ impl HttpServiceFactory for SwaggerUi {
 
 #[cfg(feature = "actix-web")]
 fn register_api_doc_url_resource(url: &str, api: OpenApi, config: &mut actix_web::dev::AppService) {
-    pub async fn get_api_doc(api_doc: web::Data<OpenApi>) -> impl Responder {
+    pub async fn get_api_doc(api_doc: web::Data<OpenApi>) -> impl ActixResponder {
         HttpResponse::Ok().json(api_doc.as_ref())
     }
 
@@ -208,6 +252,82 @@ fn register_api_doc_url_resource(url: &str, api: OpenApi, config: &mut actix_web
     HttpServiceFactory::register(url_resource, config);
 }
 
+#[cfg(feature = "rocket")]
+impl From<SwaggerUi> for Vec<Route> {
+    fn from(swagger_ui: SwaggerUi) -> Self {
+        let mut routes = Vec::<Route>::with_capacity(swagger_ui.urls.len() + 1);
+        let mut api_docs = Vec::<Route>::with_capacity(swagger_ui.urls.len());
+
+        let urls = swagger_ui.urls.into_iter().map(|(url, openapi)| {
+            api_docs.push(Route::new(
+                rocket::http::Method::Get,
+                url.url.as_ref(),
+                ServeApiDoc(openapi),
+            ));
+            url
+        });
+
+        routes.push(Route::new(
+            rocket::http::Method::Get,
+            swagger_ui.path.as_ref(),
+            ServeSwagger(swagger_ui.path.clone(), Arc::new(Config::new(urls))),
+        ));
+        routes.extend(api_docs);
+
+        routes
+    }
+}
+
+#[cfg(feature = "rocket")]
+#[derive(Clone)]
+struct ServeApiDoc(utoipa::openapi::OpenApi);
+
+#[cfg(feature = "rocket")]
+#[rocket::async_trait]
+impl Handler for ServeApiDoc {
+    async fn handle<'r>(&self, request: &'r Request<'_>, _: RocketData<'r>) -> Outcome<'r> {
+        Outcome::from(request, Json(self.0.clone()))
+    }
+}
+
+#[cfg(feature = "rocket")]
+#[derive(Clone)]
+struct ServeSwagger(Cow<'static, str>, Arc<Config<'static>>);
+
+#[cfg(feature = "rocket")]
+#[rocket::async_trait]
+impl Handler for ServeSwagger {
+    async fn handle<'r>(&self, request: &'r Request<'_>, _: RocketData<'r>) -> Outcome<'r> {
+        let mut path = self.0.as_ref();
+        if let Some(index) = self.0.find('<') {
+            path = &path[..index];
+        }
+
+        match serve(&request.uri().path().as_str()[path.len()..], self.1.clone()) {
+            Ok(swagger_file) => swagger_file
+                .map(|file| Outcome::from(request, file))
+                .unwrap_or_else(|| Outcome::from(request, NotFound("Swagger UI file not found"))),
+            Err(error) => Outcome::from(
+                request,
+                status::Custom(Status::InternalServerError, error.to_string()),
+            ),
+        }
+    }
+}
+
+#[cfg(feature = "rocket")]
+impl<'r, 'o: 'r> RocketResponder<'r, 'o> for SwaggerFile<'o> {
+    fn respond_to(self, _: &'r Request<'_>) -> rocket::response::Result<'o> {
+        rocket::response::Result::Ok(
+            Response::build()
+                .header(Header::new("Content-Type", self.content_type))
+                .sized_body(self.bytes.len(), Cursor::new(self.bytes.to_vec()))
+                .status(Status::Ok)
+                .finalize(),
+        )
+    }
+}
+
 /// Rust type for Swagger UI url configuration object.
 #[non_exhaustive]
 #[derive(Default, Clone, Debug)]
@@ -294,6 +414,15 @@ impl From<String> for Url<'_> {
     }
 }
 
+impl<'a> From<Cow<'static, str>> for Url<'a> {
+    fn from(url: Cow<'static, str>) -> Self {
+        Self {
+            url,
+            ..Default::default()
+        }
+    }
+}
+
 #[cfg(feature = "actix-web")]
 async fn serve_swagger_ui(path: web::Path<String>, data: web::Data<Config<'_>>) -> HttpResponse {
     match serve(&*path.into_inner(), data.into_inner()) {
